@@ -82,10 +82,18 @@ poisson.lvm <- function(link="log",lambda,...) {
 ###}}} poisson
 
 ###{{{ threshold
-threshold.lvm <- function(p,...) {
+
+##' @export
+threshold.lvm <- function(p,labels=NULL,...) {
     if (sum(p)>1 || any(p<0 | p>1)) stop("wrong probability vector") ;
-    function(n,...) cut(rnorm(n),breaks=c(-Inf,qnorm(cumsum(p)),Inf))
+    if (!missing(labels))
+    return(function(n,...) {
+        return(cut(rnorm(n),breaks=c(-Inf,qnorm(cumsum(p)),Inf),labels=labels))
+    })
+    function(n,...)
+        cut(rnorm(n),breaks=c(-Inf,qnorm(cumsum(p)),Inf))    
 }
+
 ###}}}
 
 ###{{{ binomial
@@ -122,7 +130,7 @@ logit.lvm <- binomial.lvm("logit")
 ##' @export
 probit.lvm <- binomial.lvm("probit")
 
-###}}} poisson
+###}}} binomial
 
 ###{{{ Gamma
 
