@@ -23,7 +23,7 @@ estimate <- function(x,...) UseMethod("estimate")
 ##'
 ##' \item{starterfun:}{Starter-function with syntax
 ##' \code{function(lvm, S, mu)}.  Three builtin functions are available:
-##' \code{startvalues}, \code{startvalues2}, \ codestartvalues3.}
+##' \code{startvalues}, \code{startvalues0}, \code{startvalues1}, ...}
 ##'
 ##' \item{estimator:}{ String defining which estimator to use (Defaults to
 ##' ``\code{gaussian}'')}
@@ -147,7 +147,7 @@ estimate <- function(x,...) UseMethod("estimate")
     start=NULL,
     constrain=lava.options()$constrain,
     method=NULL,
-    starterfun="startvalues",
+    starterfun="startvalues0",
     information="E", 
     meanstructure=TRUE,
     sparse=FALSE,
@@ -271,7 +271,8 @@ estimate <- function(x,...) UseMethod("estimate")
   } else {
     Method <- get(Method)
   }
-  if (is.null(optim$method)) optim$method <- Method
+  if (is.null(optim$method)) 
+      optim$method <- if (missing) "nlminb1" else Method
 
   if (!quick & index) {
     ## Proces data and setup some matrices
@@ -292,6 +293,8 @@ estimate <- function(x,...) UseMethod("estimate")
   if (!optim$meanstructure) {
     mu <- NULL
   }
+
+  nparall <- index(x)$npar + ifelse(optim$meanstructure, index(x)$npar.mean+index(x)$npar.ex,0)
   ## Get starting values
   myparnames <- coef(x,mean=TRUE)
   paragree <- FALSE
@@ -302,23 +305,31 @@ estimate <- function(x,...) UseMethod("estimate")
   }
   if (sum(paragree)>=length(myparnames))
     optim$start <- optim$start[which(paragree.2)]
-  
- 
+
   if (! (length(optim$start)==length(myparnames) & sum(paragree)==0)) 
   if (is.null(optim$start) || sum(paragree)<length(myparnames)) {
-    start <- suppressWarnings(do.call(optim$starterfun, list(x=x,S=S,mu=mu,debug=lava.options()$debug,silent=silent)))
+      if (is.null(optim$starterfun) && lava.options()$param!="relative")
+          optim$starterfun <- startvalues0
+      start <- suppressWarnings(do.call(optim$starterfun, list(x=x,S=S,mu=mu,debug=lava.options()$debug,silent=silent)))
+      if (!is.null(x$expar) && length(start)<nparall) {
+          ii <- which(index(x)$e1==1)
+          start <- c(start, structure(unlist(x$expar[ii]),names=names(x$expar)[ii]))
+          
+      }
     ## Debug(list("start=",start))
     if (length(paragree.2)>0) {
       start[which(paragree)] <- optim$start[which(paragree.2)]
     }
     optim$start <- start
   }
-  if (!is.null(x$expar)) optim$start <- c(optim$start,rep(0,index(x)$npar.ex))
+  coefname <- coef(x,mean=optim$meanstructure,fix=FALSE);
+  names(optim$start) <- coefname
   
   ## Missing data
   if (missing) {
-    control$start <- optim$start
-    return(estimate.MAR(x=x,data=data,fix=fix,control=control,debug=lava.options()$debug,silent=silent,estimator=estimator,weight=weight,weight2=weight2,cluster=cluster,...))
+      ##$start <- optim$start    
+      ##return(estimate.MAR(x=x,data=data,fix=fix,control=control,debug=lava.options()$debug,silent=silent,estimator=estimator,weight=weight,weight2=weight2,cluster=cluster,...))
+      return(estimate.MAR(x=x,data=data,fix=fix,control=optim,debug=lava.options()$debug,silent=silent,estimator=estimator,weight=weight,weight2=weight2,cluster=cluster,...))
   }
 
   ## Non-linear parameter constraints involving observed variables? (e.g. nonlinear regression)
@@ -607,8 +618,6 @@ estimate <- function(x,...) UseMethod("estimate")
     myInfo <- myHess <- NULL
   if (is.null(tryCatch(get(GradFun),error = function (x) NULL)))
     myGrad <- NULL
-
-  coefname <- coef(x,mean=optim$meanstructure,fix=FALSE);
   
   if (!silent) message("Optimizing objective function...")
   if (optim$trace>0 & !silent) message("\n")
@@ -623,8 +632,7 @@ estimate <- function(x,...) UseMethod("estimate")
     if (optim$constrain) {
       opt$estimate[constrained] <- exp(opt$estimate[constrained])
     }
-  ##  names(opt$estimate) <- coefname
-
+  
     if (XconstrStdOpt & !is.null(myGrad))
       opt$gradient <- as.vector(myGrad(opt$par))
     else {
@@ -632,20 +640,29 @@ estimate <- function(x,...) UseMethod("estimate")
     }
   } else {
     opt <- do.call(ObjectiveFun, list(x=x,data=data,control=control,...))
-    opt$grad <- rep(0,length(opt$estimate))
+    opt$gradient <- rep(0,length(opt$estimate))
   }
+  if (!is.null(opt$convergence)) {
+      if (opt$convergence!=0) warning("Lack of convergence. Increase number of iteration or change starting values.") 
+  } else if (!is.null(opt$gradient) && mean(opt$gradient)^2>1e-3) warning("Lack of convergence. Increase number of iteration or change starting values.") 
   if (quick) {
     return(opt$estimate)
   }
   ## Calculate std.err:
-  
+
   pp <- rep(NA,length(coefname)); names(pp) <- coefname
-  pp[names(opt$estimate)] <- opt$estimate
-  pp.idx <- na.omit(match(coefname,names(opt$estimate)))
-  
+  if (!is.null(names(opt$estimate))) {
+      pp[names(opt$estimate)] <- opt$estimate
+      pp.idx <- na.omit(match(coefname,names(opt$estimate)))
+  } else {
+      pp[] <- opt$estimate
+      pp.idx <- seq(length(pp))
+  }
+
   mom <- tryCatch(modelVar(x, pp, data=data),error=function(x)NULL)
   if (!silent) message("\nCalculating asymptotic variance...\n")
   asVarFun  <- paste(estimator, "_variance", ".lvm", sep="")
+
   if (!exists(asVarFun)) {
     if (is.null(myInfo)) {
       if (!is.null(myGrad))
@@ -667,7 +684,6 @@ estimate <- function(x,...) UseMethod("estimate")
   if (any(is.na(asVar))) {warning("Problems with asymptotic variance matrix. Possibly non-singular information matrix!")                        }
   diag(asVar)[(diag(asVar)==0)] <- NA
 
-  nparall <- index(x)$npar + ifelse(optim$meanstructure, index(x)$npar.mean+index(x)$npar.ex,0)
   mycoef <- matrix(NA,nrow=nparall,ncol=4)
   mycoef[pp.idx,1] <- opt$estimate ## Will be finished during post.hooks
   
@@ -685,7 +701,7 @@ estimate <- function(x,...) UseMethod("estimate")
               graph=NULL, control=optim)
 
   class(res) <- myclass
-  
+
   myhooks <- gethook("post.hooks")
   for (f in myhooks) {
     res0 <- do.call(f,list(x=res))
