@@ -6,7 +6,7 @@ function(x,...) UseMethod("score")
 
 ##' @S3method score lvm
 score.lvm <- function(x, data, p, model="gaussian", S, n, mu=NULL, weight=NULL, weight2=NULL, debug=FALSE, reindex=FALSE, mean=TRUE, constrain=TRUE, indiv=TRUE,...) {
-
+    
   cl <- match.call()
   lname <- paste(model,"_score.lvm",sep="")
   if (!exists(lname)) {
@@ -54,7 +54,7 @@ score.lvm <- function(x, data, p, model="gaussian", S, n, mu=NULL, weight=NULL,
       index(x0) <- reindex(x0,zeroones=TRUE,deriv=TRUE)    
     }
     pp <- modelPar(x0,p)
-    p0 <- with(pp, c(meanpar,p))
+    ##p0 <- with(pp, c(meanpar,p,p2))
     k <- length(index(x0)$manifest)
 
     myfun <- function(ii) {
@@ -62,7 +62,7 @@ score.lvm <- function(x, data, p, model="gaussian", S, n, mu=NULL, weight=NULL,
         for (i in 1:length(myfix$var)) {
           index(x0)$A[cbind(myfix$row[[i]],myfix$col[[i]])] <- data[ii,myfix$var[[i]]]
         }
-      return(scoreFun(x0,data=data[ii,], p=with(pp,c(meanpar,p)),weight=weight[ii,,drop=FALSE],weight2=weight2[ii,,drop=FALSE],model=model,debug=debug,indiv=indiv,...))
+      return(scoreFun(x0,data=data[ii,], p=with(pp,c(meanpar,p,p2)),weight=weight[ii,,drop=FALSE],weight2=weight2[ii,,drop=FALSE],model=model,debug=debug,indiv=indiv,...))
     }
     score <- t(sapply(1:nrow(data),myfun))
     if (!indiv) {
@@ -92,23 +92,25 @@ score.lvm.missing <- function(x,
                               indiv=FALSE,
                               list=FALSE,
                               ...) {
-  S <- score(x$estimate$model0, p=p, model=estimator, weight=weight,
-               indiv=indiv,...)
-  if (indiv & !list) {
-    S0 <- matrix(ncol=length(p),nrow=length(x$order))
-    rownames(S0) <- 1:nrow(S0)
-    myorder <- x$orderlist
-    if (length(x$allmis)>0)
-      myorder[[x$allmis]] <- NULL
-    for (i in 1:length(S))
-      S0[myorder[[i]],] <- S[[i]]
-    if (length(x$allmis)>0) {
-      S0 <- S0[-x$orderlist[[x$allmis]],]
+    dots <- list(...)
+    dots$combine <- FALSE
+    S <- do.call("score",c(list(x=x$estimate$model0,p=p, model=estimator, weight=weight, indiv=indiv),dots))
+    if (indiv & !list) {
+        S0 <- matrix(ncol=length(p),nrow=length(x$order))
+        rownames(S0) <- 1:nrow(S0)
+        myorder <- x$orderlist
+        if (length(x$allmis)>0)
+            myorder[[x$allmis]] <- NULL
+        for (i in 1:length(S))
+            S0[myorder[[i]],] <- S[[i]]
+        if (length(x$allmis)>0) {
+            S0 <- S0[-x$orderlist[[x$allmis]],]
+        }
+        S0[is.na(S0)] <- 0
+        colnames(S0) <- names(coef(x))
+        return(S0)
     }
-    S0[is.na(S0)] <- 0
-    return(S0)
-  }
-  return(S)
+    return(S)
 }
 
 ###}}} score.lvm.missing
@@ -125,17 +127,21 @@ score.multigroupfit <- function(x,p=pars(x), weight=Weight(x), estimator=x$estim
 ###{{{ score.multigroup
 
 ##' @S3method score multigroup
-score.multigroup <- function(x,data=x$data,weight=NULL,p,indiv=FALSE,...) {
+score.multigroup <- function(x,data=x$data,weight=NULL,weight2=NULL,p,indiv=combine,combine=FALSE,...) {
   rm <- procrandomslope(x)
   pp <- with(rm, modelPar(model,p)$p)
-  parord <- modelPar(rm$model,1:with(rm$model,npar+npar.mean))$p
+  parord <- modelPar(rm$model,seq_len(with(rm$model,npar+npar.mean)))$p
   S <- list()
-  for (i in 1:x$ngroup) {
-    S0 <- rbind(score(x$lvm[[i]],p=pp[[i]],data=data[[i]],weight=weight[[i]],indiv=indiv,...))
+  for (i in seq_len(x$ngroup)) {
+    S0 <- rbind(score(x$lvm[[i]],p=pp[[i]],data=data[[i]],weight=weight[[i]],weight2=weight2[[i]],indiv=indiv,...))
     S1 <- matrix(ncol=length(p),nrow=nrow(S0))
     S1[,parord[[i]]] <- S0
     S <- c(S, list(S1))
   }
+  if (combine) {
+      S <- Reduce("rbind",S); S[is.na(S)] <- 0
+      return(S)
+  }
   if (indiv) return(S)
   res <- matrix(0,nrow=1,ncol=length(p))
   for (i in 1:x$ngroup)
