@@ -24,8 +24,15 @@ subset.lvm <- function(x, vars, ...) {
   if (class(vars)[1]=="formula") vars <- all.vars(vars)
   if (!all(vars%in%vars(x))) stop("Not a subset of model")  
   latentvars <- intersect(vars,latent(x))
-  g0 <- subGraph(vars, Graph(x))
-  res <- graph2lvm(g0)
+  ##  g0 <- subGraph(vars, Graph(x))
+  ##  res <- graph2lvm(g0)
+  res <- lvm(vars)
+  M <- x$M[vars,vars,drop=FALSE]
+  for (i in 1:nrow(M)) {
+    if (any(M[,i]==1)) {      
+      res <- regression(res, rownames(M)[M[,i]==1], rownames(M)[i], ...)
+    }
+  }
   if (length(latentvars)>0)
     latent(res) <- latentvars
   res$cov[vars,vars] <- x$cov[vars,vars]
